@@ -35,6 +35,8 @@
   (with-current-buffer (get-buffer-create excorporate-org-buffer-name)
       (setq buffer-read-only t)
       (org-mode)
+      (use-local-map (copy-keymap org-mode-map))
+      (local-set-key "q" 'quit-window)
       (display-buffer (current-buffer))
       (let ((inhibit-read-only t))
 	(delete-region (point-min) (point-max))
@@ -88,8 +90,9 @@ are the requested participants."
   (insert (format "+ Duration: %d minutes\n"
 		  (round (/ (float-time (time-subtract end start)) 60.0))))
   (insert (format "+ Location: %s\n" location))
-  (insert "+ Invitees:\n")
-  (exco-org-insert-invitees main-invitees)
+  (when main-invitees
+    (insert "+ Invitees:\n")
+    (exco-org-insert-invitees main-invitees))
   (when optional-invitees
     (insert "+ Optional invitees:\n")
     (exco-org-insert-invitees optional-invitees)))
